Class: T::Private::Methods::DeclarationBlock

Inherits:
Struct
  • Object
show all
Defined in:
lib/types/private/methods/_methods.rb

Overview

sig_state:

  • It's a Proc if we haven't forced the thunk yet.
  • It's a Declaration if we have, but we haven't finished building the sig (This can matter for circular load-time behavior, where a method is called while its Signature is being built)
  • It's a Signature if we've finished building the sig. This allows the first-call wrapper to find its own signature even if it was evaluated externally (e.g., by run_all_sig_blocks) and @signatures_by_method was overwritten by a redefinition.

Instance Attribute Summary collapse

Instance Attribute Details

#abstractObject

Returns the value of attribute abstract

Returns:

  • (Object)

    the current value of abstract



54
55
56
# File 'lib/types/private/methods/_methods.rb', line 54

def abstract
  @abstract
end

#finalObject

Returns the value of attribute final

Returns:

  • (Object)

    the current value of final



54
55
56
# File 'lib/types/private/methods/_methods.rb', line 54

def final
  @final
end

#locObject

Returns the value of attribute loc

Returns:

  • (Object)

    the current value of loc



54
55
56
# File 'lib/types/private/methods/_methods.rb', line 54

def loc
  @loc
end

#method_nameObject

Returns the value of attribute method_name

Returns:

  • (Object)

    the current value of method_name



54
55
56
# File 'lib/types/private/methods/_methods.rb', line 54

def method_name
  @method_name
end

#modObject

Returns the value of attribute mod

Returns:

  • (Object)

    the current value of mod



54
55
56
# File 'lib/types/private/methods/_methods.rb', line 54

def mod
  @mod
end

#overridableObject

Returns the value of attribute overridable

Returns:

  • (Object)

    the current value of overridable



54
55
56
# File 'lib/types/private/methods/_methods.rb', line 54

def overridable
  @overridable
end

#overrideObject

Returns the value of attribute override

Returns:

  • (Object)

    the current value of override



54
55
56
# File 'lib/types/private/methods/_methods.rb', line 54

def override
  @override
end

#sig_stateObject

Returns the value of attribute sig_state

Returns:

  • (Object)

    the current value of sig_state



54
55
56
# File 'lib/types/private/methods/_methods.rb', line 54

def sig_state
  @sig_state
end