Class: T::Private::Methods::DeclarationBlock
- Inherits:
-
Struct
- Object
- Struct
- T::Private::Methods::DeclarationBlock
- Defined in:
- lib/types/private/methods/_methods.rb
Overview
sig_state:
- It's a
Procif we haven't forced the thunk yet. - It's a
Declarationif we have, but we haven't finished building the sig (This can matter for circular load-time behavior, where a method is called while its Signature is being built) - It's a
Signatureif we've finished building the sig. This allows the first-call wrapper to find its own signature even if it was evaluated externally (e.g., by run_all_sig_blocks) and @signatures_by_method was overwritten by a redefinition.
Instance Attribute Summary collapse
-
#abstract ⇒ Object
Returns the value of attribute abstract.
-
#final ⇒ Object
Returns the value of attribute final.
-
#loc ⇒ Object
Returns the value of attribute loc.
-
#method_name ⇒ Object
Returns the value of attribute method_name.
-
#mod ⇒ Object
Returns the value of attribute mod.
-
#overridable ⇒ Object
Returns the value of attribute overridable.
-
#override ⇒ Object
Returns the value of attribute override.
-
#sig_state ⇒ Object
Returns the value of attribute sig_state.
Instance Attribute Details
#abstract ⇒ Object
Returns the value of attribute abstract
54 55 56 |
# File 'lib/types/private/methods/_methods.rb', line 54 def abstract @abstract end |
#final ⇒ Object
Returns the value of attribute final
54 55 56 |
# File 'lib/types/private/methods/_methods.rb', line 54 def final @final end |
#loc ⇒ Object
Returns the value of attribute loc
54 55 56 |
# File 'lib/types/private/methods/_methods.rb', line 54 def loc @loc end |
#method_name ⇒ Object
Returns the value of attribute method_name
54 55 56 |
# File 'lib/types/private/methods/_methods.rb', line 54 def method_name @method_name end |
#mod ⇒ Object
Returns the value of attribute mod
54 55 56 |
# File 'lib/types/private/methods/_methods.rb', line 54 def mod @mod end |
#overridable ⇒ Object
Returns the value of attribute overridable
54 55 56 |
# File 'lib/types/private/methods/_methods.rb', line 54 def overridable @overridable end |
#override ⇒ Object
Returns the value of attribute override
54 55 56 |
# File 'lib/types/private/methods/_methods.rb', line 54 def override @override end |
#sig_state ⇒ Object
Returns the value of attribute sig_state
54 55 56 |
# File 'lib/types/private/methods/_methods.rb', line 54 def sig_state @sig_state end |